From 7a7eed27b4f09c664a763ee5771666f61fe4c950 Mon Sep 17 00:00:00 2001
From: AlexIIL <redacted>
Date: Thu, 24 Nov 2022 11:38:42 +0000
Subject: [PATCH] Replace AutoMixin's QLoader support with one that uses the
 new public api: "QuiltLoader.getModContainer(Class)". (or rather, the
 quilt-specific fabric equivalent since that returns a fabric mod container).

---
 .../unascribed/lib39/core/api/AutoMixin.java  | 20 ++++++++++++-------
 1 file changed, 13 insertions(+), 7 deletions(-)

diff --git a/src/main/java/com/unascribed/lib39/core/api/AutoMixin.java b/src/main/java/com/unascribed/lib39/core/api/AutoMixin.java
index a4c6fb1..2d8b928 100644
--- a/src/main/java/com/unascribed/lib39/core/api/AutoMixin.java
+++ b/src/main/java/com/unascribed/lib39/core/api/AutoMixin.java
@@ -5,6 +5,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.lang.invoke.MethodHandles;
 import java.lang.invoke.MethodType;
+import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URISyntaxException;
 import java.net.URL;
@@ -14,6 +15,7 @@ import java.nio.file.Path;
 import java.nio.file.spi.FileSystemProvider;
 import java.util.Collections;
 import java.util.List;
+import java.util.Optional;
 import java.util.Set;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
@@ -30,6 +32,7 @@ import com.google.common.collect.Lists;
 
 import net.fabricmc.api.EnvType;
 import net.fabricmc.loader.api.FabricLoader;
+import net.fabricmc.loader.api.ModContainer;
 
 /**
  * Extend this class and register it as your mixin plugin to autodiscover mixins inside your mixin
@@ -120,18 +123,21 @@ public class AutoMixin implements IMixinConfigPlugin {
 						}
 					}
 				} catch (FileSystemException e) {
-					if ("Cannot open an InputStream on a directory!".equals(e.getMessage())) {
-						// QLoader >= 0.18
-						//    ~ W h y ~
-						var qmfsp = Class.forName("org.quiltmc.loader.impl.filesystem.QuiltMemoryFileSystemProvider");
-						var lu = MethodHandles.privateLookupIn(qmfsp, MethodHandles.lookup());
-						FileSystemProvider fsp = (FileSystemProvider) lu.findStatic(qmfsp, "instance", MethodType.methodType(qmfsp)).invoke();
-						Path base = fsp.getPath(url.toURI());
+					FabricLoader floader = FabricLoader.getInstance();
+					// QLoader >= 0.18.1-beta.18 via api
+					try {
+						Method method = floader.getClass().getMethod("quilt_getModContainer", Class.class);
+						Optional<?> modC = (Optional<?>) method.invoke(floader, getClass());
+						Path base = ((ModContainer) modC.orElse(null)).getRootPath();
 						for (Path p : (Iterable<Path>)Files.walk(base)::iterator) {
 							if (discover(rtrn, base.relativize(p).toString(), () -> Files.newInputStream(p))) {
 								skipped++;
 							}
 						}
+					} catch (ReflectiveOperationException re) {
+						RuntimeException error = new RuntimeException("Unsupported version of quilt-loader detected - please update to 0.18.1-beta.18 or later!", re);
+						error.addSuppressed(e);
+						throw error;
 					}
 				}
 			}
-- 
2.34.1

